"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.obfuscateHtmlClassNames = void 0;
const tslib_1 = require("tslib");
const css_seasoning_1 = require("css-seasoning");
const html_escaper_1 = require("html-escaper");
const htmlparser2 = tslib_1.__importStar(require("htmlparser2"));
const utils_1 = require("../utils");
const js_1 = require("./js");
const obfuscateHtmlClassNames = ({ html, selectorConversion, obfuscateMarkerClass = "", contentIgnoreRegexes = [], }) => {
    const voidTags = [
        "area",
        "base",
        "br",
        "col",
        "command",
        "embed",
        "hr",
        "img",
        "input",
        "keygen",
        "link",
        "meta",
        "param",
        "source",
        "track",
        "wbr",
    ];
    let modifiedHtml = "";
    let insideObsClassScope = false;
    let ObsClassScopeTagCount = 0;
    let ObsClassScopeTag = "";
    let scriptContent = "";
    let isScriptTag = false;
    const usedKeys = [];
    const parser = new htmlparser2.Parser({
        onprocessinginstruction(_, data) {
            modifiedHtml += `<${data}>`;
        },
        onopentag(tagName, attribs) {
            if (tagName === "script") {
                isScriptTag = true;
                scriptContent = "";
            }
            if (attribs.class) {
                if (!insideObsClassScope &&
                    obfuscateMarkerClass &&
                    attribs.class.includes(obfuscateMarkerClass)) {
                    insideObsClassScope = true;
                    ObsClassScopeTag = tagName;
                }
                if (insideObsClassScope || !obfuscateMarkerClass) {
                    const { obfuscatedContent, usedKeys: _usedKeys } = (0, utils_1.obfuscateKeys)(selectorConversion, attribs.class, []);
                    usedKeys.push(..._usedKeys);
                    attribs.class = obfuscatedContent;
                }
            }
            if (insideObsClassScope && tagName === ObsClassScopeTag) {
                ObsClassScopeTagCount++;
            }
            modifiedHtml += `<${tagName}`;
            for (const key in attribs) {
                modifiedHtml += ` ${key}="${(0, html_escaper_1.escape)(attribs[key])}"`;
            }
            if (voidTags.includes(tagName)) {
                modifiedHtml += " />";
            }
            else {
                modifiedHtml += ">";
            }
        },
        oncomment(comment) {
            modifiedHtml += `<!--${comment}-->`;
        },
        ontext(text) {
            if (isScriptTag) {
                scriptContent += text;
            }
            else {
                modifiedHtml += text;
            }
        },
        onclosetag(tagname) {
            if (voidTags.includes(tagname)) {
                return;
            }
            if (tagname === "script" && isScriptTag) {
                isScriptTag = false;
                let obfuscatedScriptContent = scriptContent;
                Object.keys(selectorConversion).forEach((key) => {
                    const className = (0, css_seasoning_1.cssUnescape)(key).slice(1);
                    const obfuscatedJs = (0, js_1.obfuscateJs)(obfuscatedScriptContent, className, { [key]: selectorConversion[key] }, "{a HTML file path}", contentIgnoreRegexes);
                    if (obfuscatedJs !== obfuscatedScriptContent) {
                        obfuscatedScriptContent = obfuscatedJs;
                        usedKeys.push(key);
                    }
                });
                modifiedHtml += `${obfuscatedScriptContent}`;
            }
            modifiedHtml += `</${tagname}>`;
            if (insideObsClassScope && tagname === ObsClassScopeTag) {
                ObsClassScopeTagCount--;
            }
            if (ObsClassScopeTagCount === 0) {
                insideObsClassScope = false;
            }
        },
    }, { decodeEntities: true });
    parser.write(html);
    parser.end();
    return {
        obfuscatedContent: modifiedHtml,
        usedKeys: Array.from(new Set(usedKeys)),
    };
};
exports.obfuscateHtmlClassNames = obfuscateHtmlClassNames;
