"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.obfuscateCli = void 0;
const tslib_1 = require("tslib");
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const yargs_1 = tslib_1.__importDefault(require("yargs"));
const config_1 = tslib_1.__importDefault(require("./config"));
const css_1 = require("./handlers/css");
const utils_1 = require("./utils");
const obfuscate = (options) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    var _a;
    (0, utils_1.setLogLevel)(options.logLevel);
    if (!options.enable) {
        (0, utils_1.log)("info", "Obfuscation", "Obfuscation disabled");
        return;
    }
    const classConversionJsonPaths = (0, utils_1.findAllFilesWithExt)(".json", options.classConversionJsonFolderPath);
    if (options.refreshClassConversionJson &&
        classConversionJsonPaths.length > 0) {
        (0, utils_1.log)("info", "Obfuscation", "Refreshing class conversion JSON");
        for (const jsonPath of classConversionJsonPaths) {
            node_fs_1.default.unlinkSync(jsonPath);
            (0, utils_1.log)("success", "Obfuscation", `Deleted ${jsonPath}`);
        }
    }
    (0, utils_1.log)("info", "Obfuscation", "Creating/Updating class conversion JSON");
    let ignorePatterns = options.ignorePatterns;
    if (options.classIgnore) {
        if (ignorePatterns && !Array.isArray(ignorePatterns)) {
            (_a = ignorePatterns.selectors) === null || _a === void 0 ? void 0 : _a.push(...options.classIgnore);
        }
        else {
            ignorePatterns = {
                selectors: [...(ignorePatterns || []), ...options.classIgnore],
                idents: [...(ignorePatterns || [])],
            };
        }
    }
    const { conversionTables } = yield (0, css_1.obfuscateCssFiles)({
        selectorConversionJsonFolderPath: options.classConversionJsonFolderPath,
        buildFolderPath: options.buildFolderPath,
        whiteListedFolderPaths: options.whiteListedFolderPaths,
        blackListedFolderPaths: options.blackListedFolderPaths,
        mode: options.mode,
        prefix: options.prefix || { selectors: options.classPrefix },
        suffix: options.suffix || { selectors: options.classSuffix },
        ignorePatterns: ignorePatterns,
        generatorSeed: options.generatorSeed,
        removeOriginalCss: options.removeOriginalCss,
    });
    const jsonPath = node_path_1.default.join(process.cwd(), options.classConversionJsonFolderPath, "conversion.json");
    console.log({ jsonPath });
    node_fs_1.default.writeFileSync(jsonPath, JSON.stringify(conversionTables, null, 2));
    (0, utils_1.log)("success", "CSS obfuscation:", `Saved conversion table to ${(0, utils_1.getFilenameFromPath)(jsonPath)}`);
    (0, utils_1.replaceJsonKeysInFiles)({
        conversionTables: conversionTables,
        targetFolder: options.buildFolderPath,
        allowExtensions: options.allowExtensions,
        contentIgnoreRegexes: options.contentIgnoreRegexes,
        whiteListedFolderPaths: options.whiteListedFolderPaths,
        blackListedFolderPaths: options.blackListedFolderPaths,
        enableObfuscateMarkerClasses: options.enableMarkers,
        obfuscateMarkerClasses: options.markers,
        removeObfuscateMarkerClassesAfterObfuscated: options.removeMarkersAfterObfuscated,
        enableJsAst: options.enableJsAst,
    });
});
const obfuscateCli = () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const argv = yargs_1.default.option("config", {
        alias: "c",
        type: "string",
        description: "Path to the config file",
    }).argv;
    let configPath = undefined;
    if (argv.config) {
        configPath = node_path_1.default.resolve(process.cwd(), argv.config);
    }
    else {
        const configFiles = [
            "next-css-obfuscator.config.ts",
            "next-css-obfuscator.config.cjs",
            "next-css-obfuscator.config.mjs",
            "next-css-obfuscator.config.js",
        ];
        for (const file of configFiles) {
            const potentialPath = node_path_1.default.join(process.cwd(), file);
            if (node_fs_1.default.existsSync(potentialPath)) {
                configPath = potentialPath;
                break;
            }
        }
    }
    const config = new config_1.default(configPath ? require(configPath) : undefined).get();
    yield obfuscate(config);
    (0, utils_1.log)("success", "Obfuscation", "Completed~");
});
exports.obfuscateCli = obfuscateCli;
